/*
 * lamer.hpp
 *
 *  Created on: 6 Dec 2017
 *      Author: julianporter
 */

#ifndef LAMER_HPP_
#define LAMER_HPP_

#include <iostream>
#include <fstream>
#include "base.hpp"
#include "transcoder.hpp"



#endif /* LAMER_HPP_ */
